local MagicLevel = 90
local version = 10

-- version history:
-- 10	new version numbering to match LoTRO updates
--	added Critical Defence
--	introduced a MagicLevel configuration variable for level-guessing, set at 90 to suit RoR gear
--	partial German translation, thanks to Amethadan
-- 0.8  initial lotrointerface.com release, many changes from the version I posted on forums.lotro.com long ago

-- Feel free to borrow or fork code, unless you're a jerk.

import "Turbine.UI.Lotro"
import "PolymniePlugins.ItemCalcTranslate"

-- Change this from nil to "en" or "de" to override locale detection
local Language = nil

if not Language then
	Language = Turbine.Engine.GetLanguage()
	if Language == Turbine.Language.German then
		Language = "de"
	else
		Language = "en"
	end
end

local debug = false

-------------------------------------------------------------------------------
local StatOrder = {"Maximum Morale", "Maximum Power", "in-Combat Morale Regen",
	"in-Combat Power Regen", "Might", "Agility", "Vitality", "Will",
	"Fate", "Critical Rating", "Finesse Rating", "Physical Mastery Rating",
	"Tactical Mastery Rating", "Resistance Rating", "Critical Defence", 
	"Incoming Healing Rating", "Block Rating", "Parry Rating",
	"Evade Rating", "Physical Mitigation", "Tactical Mitigation"}

local QualityNames = {"uncommon", "rare", "incomparable", "epic"}
local CategoryNames = {"light armour", "medium armour", "heavy armour", "cloak",
	"jewellery", "shield", "melee 1h", "melee 2h", "ranged"}
local ArmourNames = {"head", "shoulders", "chest", "hands", "legs", "feet"}
local JewelleryNames = {"bracelet", "earring", "necklace", "pocket", "ring"}
local ShieldNames = {"light", "warden", "heavy"}
local Melee1Names = {"axe", "club", "dagger", "hammer", "mace", "spear", "sword"}
local Melee2Names = {"axe", "club", "halberd", "hammer", "rune-stone", "staff", "sword"}
local RangedNames = {"bow", "crossbow", "javelin"}

local QualityColours = { -- http://content.turbine.com/sites/playerportal/modules/lotro-lorebook/css/lorebook.css
	Turbine.UI.Color(0xff / 255, 0xff / 255, 0x00 / 255),
	Turbine.UI.Color(0xc2 / 255, 0x00 / 255, 0xc7 / 255),
	Turbine.UI.Color(0x00 / 255, 0xfe / 255, 0xff / 255),
	Turbine.UI.Color(0xf2 / 255, 0xab / 255, 0x0c / 255)}

-------------------------------------------------------------------------------
local Depth = 30
local RowHeight = 20
local WindowWidth = 380
local WindowHeight = 580

CalcWindow = Turbine.UI.Lotro.Window()
CalcWindow:SetPosition((Turbine.UI.Display:GetWidth() - WindowWidth) / 2, 0)
CalcWindow:SetSize(WindowWidth, WindowHeight)
CalcWindow:SetText("ItemCalc "..version)
CalcWindow:SetVisible(true)

function MakeLabel(Left, Top, Width, Height, Text, Alignment)
	local TempLabel = Turbine.UI.Label()
	TempLabel:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempLabel:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1))
	TempLabel:SetParent(CalcWindow)
	TempLabel:SetPosition(Left, Top)
	TempLabel:SetSize(Width, Height)
	TempLabel:SetMouseVisible(false)
	if Text then
		TempLabel:SetText(Text)
	end

	if Alignment then
		TempLabel:SetTextAlignment(Alignment)
	end

	return TempLabel
end

function MakeTextbox(Left, Top, Width, Height)
	local TempTextbox = Turbine.UI.Lotro.TextBox()
	TempTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempTextbox:SetParent(CalcWindow)
	TempTextbox:SetPosition(Left, Top)
	TempTextbox:SetSize(Width, Height)
	TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)

	TempTextbox.FocusGained = function()
		TempTextbox:SetBackColor(Turbine.UI.Color.Blue)
		TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
	end
	TempTextbox.FocusLost = function()
		TempTextbox:SetBackColor(Turbine.UI.Color.Black)
		TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)
		Calculate()
	end

	return TempTextbox
end

function MakeButton(Left, Top, Width, Height, Text)
	local TempButton = Turbine.UI.Lotro.Button()
	TempButton:SetParent(CalcWindow)
	TempButton:SetPosition(Left, Top)
	TempButton:SetSize(Width, Height)
	TempButton:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempButton:SetText(Text)
	return TempButton
end

local Rows = {}

function MakeRow(stat)
	Rows[stat] = {}

	Rows[stat].name = MakeLabel(20, Depth, 170, RowHeight - 1, stat)
	Rows[stat].input = MakeTextbox(190, Depth, 50, RowHeight - 1)
	Rows[stat].typical = MakeLabel(250, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)
	Rows[stat].unit = MakeLabel(310, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)

	Depth = Depth + RowHeight
end

for i, v in ipairs(StatOrder) do
	MakeRow(v)
end

local LevelLabel = MakeLabel(20, Depth, 140, RowHeight - 1)
local UnitTotalLabel = MakeLabel(310, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)

Depth = Depth + RowHeight
local OverrideLabel = MakeLabel(20, Depth, 100, RowHeight - 1, "level override:")
local OverrideBox = MakeTextbox(120, Depth, 50, RowHeight - 1)
local UnitMaxLabel = MakeLabel(310, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)

local ResetButton = MakeButton(190, Depth, 50, RowHeight, "reset")
ResetButton.Click = function()
	ResetWindow()
end

Depth = Depth + RowHeight
local QualityButton = MakeButton(20, Depth, 100, RowHeight, QualityNames[2]) -- rare by default
QualityButton:SetForeColor(QualityColours[2]) -- rare by default
local QualityContext = Turbine.UI.ContextMenu()
local QualityContextItems = QualityContext:GetItems()
for i, v in ipairs(QualityNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		QualityButton:SetText(v)
		QualityButton:SetForeColor(QualityColours[i])
		Calculate()
	end
	QualityContextItems:Add(menuItem)
end
QualityButton.Click = function()
	QualityContext:ShowMenu()
end

local SubtypeButton = MakeButton(230, Depth, 90, RowHeight, ArmourNames[1])
local SubtypeContext
SubtypeButton.Click = function()
	SubtypeContext:ShowMenu()
end

local ArmourContext = Turbine.UI.ContextMenu()
local ArmourContextItems = ArmourContext:GetItems()
for i, v in ipairs(ArmourNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	ArmourContextItems:Add(menuItem)
end

SubtypeContext = ArmourContext -- default

local JewelleryContext = Turbine.UI.ContextMenu()
local JewelleryContextItems = JewelleryContext:GetItems()
for i, v in ipairs(JewelleryNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	JewelleryContextItems:Add(menuItem)
end

local ShieldContext = Turbine.UI.ContextMenu()
local ShieldContextItems = ShieldContext:GetItems()
for i, v in ipairs(ShieldNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	ShieldContextItems:Add(menuItem)
end

local Melee1Context = Turbine.UI.ContextMenu()
local Melee1ContextItems = Melee1Context:GetItems()
for i, v in ipairs(Melee1Names) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	Melee1ContextItems:Add(menuItem)
end

local Melee2Context = Turbine.UI.ContextMenu()
local Melee2ContextItems = Melee2Context:GetItems()
for i, v in ipairs(Melee2Names) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	Melee2ContextItems:Add(menuItem)
end

local RangedContext = Turbine.UI.ContextMenu()
local RangedContextItems = RangedContext:GetItems()
for i, v in ipairs(RangedNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		SubtypeButton:SetText(v)
		Calculate()
	end
	RangedContextItems:Add(menuItem)
end

local CategoryButton = MakeButton(120, Depth, 110, RowHeight, CategoryNames[1])
local CategoryContext = Turbine.UI.ContextMenu()
local CategoryContextItems = CategoryContext:GetItems()
for i, v in ipairs(CategoryNames) do
	local menuItem = Turbine.UI.MenuItem(v)
	menuItem.Click = function()
		CategoryButton:SetText(v)
		if v == "light armour"
		or v == "medium armour"
		or v == "heavy armour" then
			SubtypeContext = ArmourContext
			SubtypeButton:SetText(ArmourNames[1])
			SubtypeButton:SetVisible(true)
		elseif v == "jewellery" then
			SubtypeContext = JewelleryContext
			SubtypeButton:SetText(JewelleryNames[1])
			SubtypeButton:SetVisible(true)
		elseif v == "shield" then
			SubtypeContext = ShieldContext
			SubtypeButton:SetText(ShieldNames[1])
			SubtypeButton:SetVisible(true)
		elseif v == "melee 1h" then
			SubtypeContext = Melee1Context
			SubtypeButton:SetText(Melee1Names[1])
			SubtypeButton:SetVisible(true)
		elseif v == "melee 2h" then
			SubtypeContext = Melee2Context
			SubtypeButton:SetText(Melee2Names[1])
			SubtypeButton:SetVisible(true)
		elseif v == "ranged" then
			SubtypeContext = RangedContext
			SubtypeButton:SetText(RangedNames[1])
			SubtypeButton:SetVisible(true)
		elseif v == "cloak" then
			SubtypeButton:SetText(nil)
			SubtypeButton:SetVisible(false)
		end
		Calculate()
	end
	CategoryContextItems:Add(menuItem)
end
CategoryButton.Click = function()
	CategoryContext:ShowMenu()
end

Depth = Depth + RowHeight
local ExtraLabel = MakeLabel(20, Depth, WindowWidth - 40, WindowHeight - Depth - 20)

-------------------------------------------------------------------------------
function round(number, zeroes)
	local zeroes = zeroes or 0
	return math.floor(number * (10 ^ zeroes) + .5) / (10 ^ zeroes)
end

-------------------------------------------------------------------------------
function CloserToMagicLevel(new, old)
	if math.abs(MagicLevel - new) < math.abs(MagicLevel - old) then
		return true
	end
end

function TypicalRatingsByLevel(stat, level)
	local typical
	if stat == "Maximum Morale" then
		if level > 49 then
			typical = level * 3.5 - 125.5
		else
			typical = level * 0.9 + 0.9
		end
	elseif stat == "Maximum Power" then
		if level > 49 then
			typical = level * 3 - 104
		else
			typical = level * 0.9 + 0.9
		end
	elseif stat == "in-Combat Morale Regen" then
		if level > 49 then
			typical = level * 0.9 - 14.1
		else
			typical = level * 0.6 + 0.6
		end
	elseif stat == "in-Combat Power Regen" then
		if level > 49 then
			typical = level * 0.3 + 30.3
		else
			typical = level * 0.9 + 0.9
		end
	elseif stat == "Might"
	or stat == "Agility"
	or stat == "Vitality"
	or stat == "Will"
	or stat == "Fate" then
		if level > 59 then
			typical = level * 0.5 - 10.5
		elseif level > 49 then
			typical = level * 0.3 + 1.3
		else
			typical = level * 0.3 + 0.1
		end
	elseif stat == "Critical Rating" then
		typical = level * 4
	elseif stat == "Finesse Rating" then
		if level > 51 then
			typical = level * 12 + 12
		elseif level < 50 then
			typical = level * 12 + 48
		else
			typical = 636
		end
	elseif stat == "Physical Mastery Rating" then
		typical = level * 4
	elseif stat == "Tactical Mastery Rating" then
		typical = level * 4
	elseif stat == "Resistance Rating" then
		typical = level * 6
	elseif stat == "Critical Defence" then
		typical = level * 10 + 10
	elseif stat == "Incoming Healing Rating" then
		typical = level * 8
	elseif stat == "Block Rating" then
		if CategoryButton:GetText() == "shield" then -- ugly and ominous
			typical = level * 6
		else
			typical = level * 4
		end
	elseif stat == "Parry Rating" then
		typical = level * 4
	elseif stat == "Evade Rating" then
		typical = level * 4
	elseif stat == "Physical Mitigation" then
		typical = level * 8.1 + 0.4
	elseif stat == "Tactical Mitigation" then
		typical = level * 8.1 + 0.4
	end

	if typical < 1 then  -- avoid later div-by-0 and other oddities
		return 1
	else
		return typical
	end
end

function GuessLevel(stat, rating)
	local slope, ambiguous
	local constant = 0
	if stat == "Maximum Morale" then
		slope = 3.5
		constant = - 125.5
	elseif stat == "Maximum Power" then
		slope = 3
		constant = - 104
	elseif stat == "in-Combat Morale Regen" then
		slope = 0.9
		constant = - 14.1
	elseif stat == "in-Combat Power Regen" then
		slope = 0.3
		constant = 30.3
		ambiguous = true
	elseif stat == "Might"
	or stat == "Agility"
	or stat == "Vitality"
	or stat == "Will"
	or stat == "Fate" then
		slope = 0.5
		constant = - 10.5
		ambiguous = true
	elseif stat == "Critical Rating" then
		slope = 4
	elseif stat == "Finesse Rating" then
		slope = 12
		constant = 12
	elseif stat == "Physical Mastery Rating" then
		slope = 4
	elseif stat == "Tactical Mastery Rating" then
		slope = 4
	elseif stat == "Resistance Rating" then
		slope = 6
	elseif stat == "Critical Defence" then
		slope = 10
		constant = 10
	elseif stat == "Incoming Healing Rating" then
		slope = 8
	elseif stat == "Block Rating" then
		if CategoryButton:GetText() == "shield" then -- ugly and ominous
			slope = 6
		else
			slope = 4
		end
	elseif stat == "Parry Rating" then
		slope = 4
	elseif stat == "Evade Rating" then
		slope = 4
	elseif stat == "Physical Mitigation" then
		slope = 8.1
		constant = 0.4
	elseif stat == "Tactical Mitigation" then
		slope = 8.1
		constant = 0.4
	end

	local BestGuess, NewGuess, mult
	for i = 1, 4 do
		NewGuess = math.floor(((rating / i) - constant) / slope)

		-- assume the mult that leads closest to 85 is correct
		-- I may need to change this as expansions raise the typical item range
		if not BestGuess or CloserToMagicLevel(NewGuess, BestGuess) then
			BestGuess = NewGuess
			mult = i
		end
	end

	if ambiguous and (mult == 1 or mult == 3) then -- don't trust
		if debug then
			Turbine.Shell.WriteLine(string.format("[calc] ((%s %s / %d) - %s) / %s = %s?",
				rating, stat, mult, constant, slope, BestGuess))
		end
		return BestGuess, false
	else -- trust, but I could be wrong
		if debug then
			Turbine.Shell.WriteLine(string.format("[calc] ((%s %s / %d) - %s) / %s = %s",
				rating, stat, mult, constant, slope, BestGuess))
		end
		return BestGuess, true
	end
end

-------------------------------------------------------------------------------
function GuessArmour(level)
	local quality = QualityButton:GetText()

	-- insufficient data to implement epics
	-- epic armour is approx legendary(level)/0.95
	-- or epic armour is approx legendary(level + 5)
	if quality == "epic" then
		return "?"
	end

	local multiplier
	local category = CategoryButton:GetText()
	local subtype = SubtypeButton:GetText()

	if subtype == "chest" then
		multiplier = 30
	elseif subtype == "legs" then
		multiplier = 25
	elseif subtype == "hands" then
		multiplier = 15
	elseif subtype == "feet" then
		multiplier = 12
	elseif subtype == "head" then
		multiplier = 9
	elseif subtype == "shoulders" then
		multiplier = 9
	end

	if category == "shield" then
		if subtype == "light" then
			multiplier = 388 -- FIXME
		elseif subtype == "warden" then
			multiplier = 564 -- FIXME
		else
			multiplier = 1250
		end
	elseif category == "cloak" then
		multiplier = 220
	elseif category == "heavy armour" then
		multiplier = multiplier * 22
	elseif category == "medium armour" then
		multiplier = multiplier * 17
	else
		multiplier = multiplier * 12
	end

	multiplier = multiplier * 27 / 1000

	-- shields are frustrating
	if category == "shield" and subtype == "light" then
		if quality == "incomparable" then
			return round(level * multiplier) -- FIXME
		elseif quality == "rare" then
			return round((level - 2) * multiplier) -- correct
		else
			return round(level * multiplier * 0.785) -- FIXME
		end
	elseif category == "shield" and subtype == "heavy" then
		if quality == "incomparable" then
			return round(level * multiplier + 0.4)
		elseif quality == "rare" then
			return round(round(level * multiplier + 0.4) * 0.8)
		else
			return round(round((level - 3) * multiplier + 0.4) * 0.8)
		end
	else -- warden shields and everything else
		if quality == "incomparable" then
			return round(level * multiplier)
		elseif quality == "rare" then
			return round(round(level * multiplier) * 0.95)
		else
			return round(round((level - 3) * multiplier) * 0.95)
		end
	end
end

-------------------------------------------------------------------------------
-- FIXME an absolute travesty
local dpsLookup = {
	["uncommon"] = {
		-- melee 1h
		{  2.6,   nil,   nil,   nil,   5.2,   5.9,   6.6,   7.2,   7.9,   8.5,
		   9.2,   9.8,  10.5,  11.1,  11.8,  12.5,  13.1,  13.8,  14.4,  15.1,
		  15.7,  16.4,  17.0,  17.7,  18.4,  19.0,  19.7,  20.3,  21.0,  21.6,
		  22.3,  22.9,  23.6,  24.3,  24.9,  25.6,  26.2,  26.9,  27.5,  28.2,
		  28.8,  29.5,  30.2,  30.8,  31.5,  32.1,  32.8,  33.4,  34.1,  34.7,
		   nil,  36.6,   nil,   nil,  41.5,   nil,  45.2,   nil,   nil,   nil,
		  51.4,  52.7,  54.0,  55.4,  58.1,  61.0,  64.1,  67.1,  70.2,  73.2,
		  76.3,  79.3,  82.4,  85.4,  88.5},
		-- melee 2h, ranged
		{  3  ,   nil,   nil,   nil,   6  ,   6.7,   7.5,   8.2,   9  ,   9.7,
		  10.5,  11.2,  12  ,  12.7,  13.5,  14.2,  14.9,  15.7,  16.4,  17.2,
		  17.9,  18.7,  19.4,  20.2,  20.9,  21.7,  22.4,  23.2,  23.9,  24.7,
		  25.4,  26.2,  26.9,  27.6,  28.4,  29.1,  29.9,  30.6,  31.4,  32.1,
		  32.9,  33.6,  34.4,  35.1,  35.9,  36.6,  37.4,  38.1,  38.9,  39.6,
		   nil,  41.5,   nil,   nil,  47  ,   nil,  51.3,   nil,   nil,   nil,
		  58.3,  59.7,  61.2,  62.7,  65.9,  69.2,  72.6,  76.1,  79.5,  83  ,
		  86.4,  89.9,  93.4,  96.8, 100.3}
	},

	["rare"] = {
		-- melee 1h
		{  2.8,   nil,   nil,   nil,   nil,   nil,   nil,   7.7,   8.4,   9.1,
		   9.8,  10.5,  11.2,  11.9,  12.6,  13.3,  14  ,  14.7,  15.4,  16.1,
		  16.8,  17.5,  18.2,  18.9,  19.6,  20.3,  21  ,  21.7,  22.4,  23.1,
		  23.8,  24.5,  25.2,  25.9,  26.6,  27.3,  28  ,  28.7,  29.4,  30.1,
		  30.8,  31.5,  32.2,  32.9,  33.6,  34.3,  35  ,  35.7,  36.4,  37.1,
		  38  ,  39  ,  40.1,  42.1,  43.2,  45.2,  47.3,  50.1,  52  ,  53.4,
		  54.8,  56.2,  57.6,  59  ,  62  ,  64  ,  66.8,  70  ,  72.8,  77.1,
		  79.3,  83.6,  87.9,  91.1,  94.4,  97.6, 100.9, 104.1,   nil,   nil,
		   nil, 117.1,   nil,   nil,   nil,   nil,   nil, 129.6,   nil,   nil,
		   nil,   nil,   nil, 134.1},
		-- melee 2h, ranged
		{  3.2,   nil,   nil,   nil,   nil,   nil,   nil,   8.8,   9.6,  10.4,
		  11.2,  12  ,  12.8,  13.6,  14.3,  15.1,  15.9,  16.7,  17.5,  18.3,
		  19.1,  19.9,  20.7,  21.5,  22.3,  23.1,  23.9,  24.7,  25.5,  26.3,
		  27.1,  27.9,  28.7,  29.5,  30.3,  31.1,  31.9,  32.7,  33.5,  34.3,
		  35.1,  35.9,  36.7,  37.5,  38.3,  39.1,  39.9,  40.7,  41.4,  42.2,
		  43  ,  44.2,  45.5,  47.8,  50.1,  52.7,  54.7,  56.8,  59  ,  60.6,
		  62.1,  63.7,  65.3,  66.9,  70.3,  73.8,  77.5,  81.1,  84.8,  88.5,
		  92.2,  95.9,  99.6, 103.3, 107  , 110.6, 114.3, 118  ,   nil,   nil,
		   nil, 132.8,   nil,   nil,   nil,   nil,   nil, 146.7,   nil,   nil,
		   nil,   nil,   nil, 151.2}
	},

	["incomparable"] = {
		-- melee 1h
		{  nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
		   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
		   nil,   nil,   nil,   nil,   nil,   nil,  21.4,  22.2,  22.9,  23.7,
		  24.5,  25.2,  26  ,  26.8,  27.5,  28.3,  29.1,  29.8,  30.6,  31.4,
		  32.1,  32.9,  33.6,  34.4,  35.2,  35.9,  36.7,  37.5,  38.2,  39  ,
		  39.8,  40.5,  41.5,  42.7,  43.9,  46.1,  48.4,  50.8,  52.8,  54.8,
		  56.9,  58.4,  60  ,  61.5,  63  ,  64.6,  67.8,  71.2,  74.8,  78.3,
		  81.9,  85.4,  89  ,  92.5,  96.1,  99.6, 103.2, 106.8, 110.3, 113.9,
		 117.4, 121  ,   nil, 128.1,   nil,   nil,   nil,   nil,   nil, 132.6,
		   nil,   nil,   nil,   nil, 135.6, 137.1},
		-- melee 2h, ranged
		{  nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
		   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
		   nil,   nil,   nil,   nil,   nil,   nil,  24.8,  25.7,  26.5,  27.3,
		  28.2,  29  ,  29.8,  30.6,  31.5,  32.3,  33.1,  34  ,  34.8,  35.6,
		  36.4,  37.3,  38.1,  38.9,  39.8,  40.6,  41.4,  42.2,  43.1,  43.9,
		  44.7,  46  ,  47.3,  49.6,  52.1,  54.7,  56.8,  59  ,  61.3,  62.9,
		  64.6,  66.2,  67.9,  69.5,  73  ,  76.7,  80.5,  84.3,  88.1,  92  ,
		  95.8,  99.6, 103.5, 107.3, 111.1, 115  , 118.8, 122.6, 126.5, 130.3,
		 134.1, 137.9,   nil, 145.6,   nil,   nil,   nil,   nil,   nil, 149.7,
		   nil,   nil,   nil,   nil,   nil, 154.2}
	},

	["epic"] = {
		-- melee 1h
		{},
		-- melee 2h, ranged
		{}
	},
}

function GuessDamage(level)
	local quality = QualityButton:GetText()

	local hands = 1
	local category = CategoryButton:GetText()
	if category == "melee 2h" or category == "ranged" then
		hands = 2
	end

	local dps = dpsLookup[quality][hands][level]
	if not dps then
		return "\ndamage prediction: unknown (FIXME)"
	end

	-- melee 1h: avg strike = dps * 1.9
	-- melee 2h: avg strike = dps * 2.5
	-- javelin : avg strike = dps * 2.5
	-- bow     : avg strike = dps * 2.2

	-- melee 1h: spread = avg strike ± 25%
	-- melee 2h: spread = avg strike ± 25%
	-- javelin : spread = avg strike ± 25%
	-- bow     : spread = avg strike ± 29%

	local min, max
	if category == "melee 1h" then
		min = round(dps * 1.9 * 0.75)
		max = round(dps * 1.9 * 1.25)
	elseif category == "melee 2h" then
		min = round(dps * 2.5 * 0.75)
		max = round(dps * 2.5 * 1.25)
	elseif category == "ranged" then
		if SubtypeButton:GetText() == "javelin" then
			min = round(dps * 2.5 * 0.75)
			max = round(dps * 2.5 * 1.25)
		else
			min = round(dps * 2.2 * 0.71)
			max = round(dps * 2.2 * 1.29)
		end
	end

	return string.format("\ndamage prediction: %s DPS\n(%s - %s Damage)", dps, min, max)
end

-------------------------------------------------------------------------------
function GuessVendor(level)
	local copper
	local category = CategoryButton:GetText()
	local subtype = SubtypeButton:GetText()

	if category == "light armour" then
		if subtype == "chest" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 76
		elseif subtype == "legs" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 60
		elseif subtype == "head" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 50
		elseif subtype == "hands" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 40
		elseif subtype == "feet" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 40
		elseif	subtype == "shoulders" then
			copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 36
		end
	elseif category == "medium armour" then
		if subtype == "chest" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 83
		elseif subtype == "legs" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 67
		elseif subtype == "head" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 55
		elseif subtype == "hands" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 45
		elseif subtype == "feet" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 45
		elseif subtype == "shoulders" then
			copper = math.min(level, 50) * 16.5 + math.max(level - 50, 0) * 16 + 39
		end
	elseif category == "heavy armour" then
		if subtype == "chest" then
			copper = math.min(level, 50) * 21.5 + math.max(level - 50, 0) * 18 + 107
		elseif subtype == "legs" then
			copper = math.min(level, 50) * 21.5 + math.max(level - 50, 0) * 18 + 85
		elseif subtype == "head" then
			copper = math.min(level, 50) * 20 + math.max(level - 50, 0) * 18 + 56
		elseif subtype == "hands" then
			copper = math.min(level, 50) * 20 + math.max(level - 50, 0) * 18 + 44
		elseif subtype == "feet" then
			copper = math.min(level, 50) * 20 + math.max(level - 50, 0) * 18 + 44
		elseif subtype == "shoulders" then
			copper = math.min(level, 50) * 20 + math.max(level - 50, 0) * 18 + 38
		end
	elseif category == "cloak" then
		copper = math.min(level, 50) * 15 + math.max(level - 50, 0) * 16 + 0
	elseif category == "jewellery" then
		if subtype == "bracelet" then
			copper = math.min(level, 50) * 5 + math.max(level - 50, 0) * 6 + 50
		elseif subtype == "earring" then
			copper = math.min(level, 50) * 5 + math.max(level - 50, 0) * 6 + 0
		elseif subtype == "necklace" then
			copper = math.min(level, 50) * 5 + math.max(level - 50, 0) * 6 + 150
		elseif subtype == "ring" then
			copper = math.min(level, 50) * 5 + math.max(level - 50, 0) * 6 + 50
		elseif subtype == "pocket" then
			copper = math.min(level, 50) * 22 + math.max(level - 50, 0) * 24 + 82
		end
	elseif category == "shield" then
		if subtype == "light" then
			copper = math.min(level, 50) * 11 + math.max(level - 50, 0) * 12 + 110
		elseif subtype == "warden" then
			copper = math.min(level, 50) * 12 + math.max(level - 50, 0) * 12 + 110
		elseif subtype == "heavy" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 560
		end
	elseif category == "melee 1h" then
		if subtype == "axe" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 150
		elseif subtype == "club" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 148
		elseif subtype == "dagger" then
			copper = math.min(level, 50) * 8 + math.max(level - 50, 0) * 10 + 130
		elseif subtype == "hammer" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 148
		elseif subtype == "mace" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 148
		elseif subtype == "spear" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 50
		elseif subtype == "sword" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 150
		end
	elseif category == "melee 2h" then
		if subtype == "axe" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 560
		elseif subtype == "club" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 560
		elseif subtype == "halberd" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 200
		elseif subtype == "hammer" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 560
		elseif subtype == "rune-stone" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 100
		elseif subtype == "staff" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 100
		elseif subtype == "sword" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 560
		end
	elseif category == "ranged" then
		if subtype == "bow" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 120
		elseif subtype == "crossbow" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 500
		elseif subtype == "javelin" then
			copper = math.min(level, 50) * 14 + math.max(level - 50, 0) * 14 + 120
		end
	end

	local quality = QualityButton:GetText()
	if quality == "uncommon" then
		copper = copper * 0.6
	elseif quality == "incomparable" then
		copper = copper * 1.5
	elseif quality == "epic" then
		copper = copper * 2
	end

	local output = "vendor prediction:"
	local silver = math.floor(copper / 100)
	if silver > 0 then
		copper = copper - (silver * 100)
		output = string.format("%s %ss", output, silver)
	end
	if copper ~= 0 then
		output = string.format("%s %dc", output, copper) -- forced to integer
	end

	return output
end

-------------------------------------------------------------------------------
function Calculate()
	-- determine valid inputs and best-guess of level
	local ValidInputs = {}
	local best = 1
	local trustbest = false

	for k, v in pairs(Rows) do
		local rating = tonumber(v.input:GetText())
		if rating and rating ~= 0 then
			ValidInputs[k] = rating
			v.input:SetText(rating) -- fix whitespace problems
			local guess, trustguess = GuessLevel(k, rating)
			if trustguess and (not trustbest or CloserToMagicLevel(guess, best)) then
				best = guess
				trustbest = true
			elseif not trustbest and CloserToMagicLevel(guess, best) then
				best = guess
			end
		else
			v.input:SetText(nil) -- clear bad inputs
			v.unit:SetText(nil)
		end
	end

	LevelLabel:SetText("suspected level: "..best)

	if best < 66 then
		LevelLabel:SetForeColor(Turbine.UI.Color.Red)
	else
		LevelLabel:SetForeColor(Turbine.UI.Color.Lime)
	end

	local override = tonumber(OverrideBox:GetText())
	if override and override > 0 then
		best = override
		OverrideBox:SetText(override) -- fix whitespace problems
	else
		OverrideBox:SetText(nil)
	end

	-- calculate units, equivs, and totals
	local UnitTotal = 0

	for k, v in pairs(Rows) do
		local typical = TypicalRatingsByLevel(k, best)
		v.typical:SetText(typical)
		if ValidInputs[k] then
			local unit = ValidInputs[k] / typical
			UnitTotal = UnitTotal + unit
			v.unit:SetText(unit)
		end
	end

	-- calculate budget size
	UnitTotalLabel:SetText(UnitTotal)

	local quality = QualityButton:GetText()
	local QualitySize

	if quality == "epic" then
		QualitySize = 8
	elseif quality == "incomparable" then
		QualitySize = 7
	elseif quality == "rare" then
		QualitySize = 6
	else
		QualitySize = 5
	end

	local category = CategoryButton:GetText()
	if category == "melee 2h" then
		QualitySize = QualitySize * 2
	end

	UnitMaxLabel:SetText("of "..QualitySize)

	-- provide vendor value, damage, armour, crit defence details
	local extra = GuessVendor(best)
	if category == "melee 1h" or category == "melee 2h" or category == "ranged" then
		extra = extra..GuessDamage(best)
	elseif category ~= "jewellery" then
		extra = extra.."\narmour prediction: "..GuessArmour(best)
		if category == "shield" then
			local subtype = SubtypeButton:GetText()
			if subtype == "heavy" then
				extra = extra.."\ncritical defence: "..(best + 1) * 20
			elseif subtype == "warden" then
				extra = extra.."\ncritical defence: "..(best + 1) * 16
			else
				extra = extra.."\ncritical defence: "..(best + 1) * 8
			end
		end
	end
	ExtraLabel:SetText(extra)
end

-------------------------------------------------------------------------------
function ResetWindow()
	for k, v in pairs(Rows) do
		v.input:SetText(nil)
	end
	OverrideBox:SetText(nil)
	Calculate()
end

-------------------------------------------------------------------------------
function UpdateLanguage()
	for k, v in pairs(Rows) do
		v.name:SetText(translate(k, Language))
	end
end

-------------------------------------------------------------------------------
local CommandLine = Turbine.ShellCommand()
CommandLine.Execute = function(sender, cmd, args)
	if args == "toggle" and CalcWindow:IsVisible() then
		CalcWindow:SetVisible(false)
		return
	else
		CalcWindow:SetVisible(true)
	end

	if args == "reset" then
		ResetWindow()
		return
	end

	if args == "debug" then
		if debug then
			debug = false
			Turbine.Shell.WriteLine("DreadMeter debugging off.")
		else
			debug = true
			Turbine.Shell.WriteLine("DreadMeter debugging on.")
		end
	elseif args == "german" then
		Language = "de"
		UpdateLanguage()
	elseif args == "english" then
		Language = "en"
		UpdateLanguage()
	end

	Calculate()
end

Turbine.Shell.AddCommand("ItemCalc", CommandLine)

-------------------------------------------------------------------------------
-- create a dummy window, and update until loaded, at which point set loaded=true and set the unload commands.
local loaded = false
tmpWindow = Turbine.UI.Window()
tmpWindow.Update = function()
	if Plugins["ItemCalc"] ~= nil and not loaded then
		loaded = true
		Plugins["ItemCalc"].Unload = function()
			Turbine.Shell.RemoveCommand(CommandLine)
		end
		tmpWindow:SetWantsUpdates(false)
		Turbine.Shell.WriteLine("Polymnie's ItemCalc plugin loaded.")
		ResetWindow()
	end
end
tmpWindow:SetWantsUpdates(true)
